@@ -1,22 +1,25 @@
 import logging
 
 from bitfield import BitField, BitHandler
+from django.conf import settings
 from django.db import transaction
 from django.db.models import PositiveIntegerField, BigIntegerField, UniqueConstraint, Q
 from django.forms import model_to_dict
 
 from isc_common import setAttr, delAttr
 from isc_common.bit import TurnBitOn
+from isc_common.common.mat_views import refresh_mat_view
 from isc_common.fields.related import ForeignKeyProtect
 from isc_common.http.DSRequest import DSRequest
 from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
 from isc_common.models.audit import AuditModel
-from isc_common.progress import managed_progress
+from isc_common.progress import managed_progress, ProgressDroped
 from isc_common.string import get_NoneStr
 from kaf_pas.ckk.models import get_operations_from_trunsaction
-from kaf_pas.kd.models.document_attributes import Document_attributes
+from kaf_pas.kd.models.document_attributes import Document_attributes, Document_attributesManager
 from kaf_pas.kd.models.documents import Documents
 from kaf_pas.kd.models.lotsman_documents_hierarcy import Lotsman_documents_hierarcy
+from kaf_pas.production.models import progress_deleted
 
 logger = logging.getLogger(__name__)
 
@@ -56,9 +59,6 @@ class ItemManager(CommonManagetWithLookUpFieldsManager):
         res = super().create(**_data)
         logger.debug(f'Added: {res}')
 
-        # if not data.get('parent_id'):
-        #     raise Exception('Не выбран родительский узел.')
-
         item_refs, create = Item_refs.objects.get_or_create(parent_id=_data.get('parent_id'), child=res)
         if create:
             logger.debug(f'Added item_ref: {item_refs}')
@@ -74,71 +74,86 @@ class ItemManager(CommonManagetWithLookUpFieldsManager):
         return data
 
     @staticmethod
-    def delete_recursive(item_id, delete_lines=False, soft_delete=False, user=None):
-        with transaction.atomic():
-            from isc_common.management.commands.refresh_mat_views import refresh_mat_view
-            from kaf_pas.ckk.models.item_image_refs import Item_image_refs
-            from kaf_pas.ckk.models.item_line import Item_line
-            from kaf_pas.ckk.models.item_refs import Item_refs
-            from kaf_pas.production.models.operations_item import Operations_item
-            from django import db
+    def delete_recursive(item_id, delete_lines=False, soft_delete=False, user=None, props=TurnBitOn(0, 0), document=None):
+        from django import db
+        from isc_common.models.deleted_progresses import Deleted_progresses
+        from kaf_pas.ckk.models.item_image_refs import Item_image_refs
+        from kaf_pas.ckk.models.item_line import Item_line
+        from kaf_pas.ckk.models.item_refs import Item_refs
+        from kaf_pas.production.models.operations_item import Operations_item
 
-            item_lotsman_document_cnt = 0
-            cnt = Item_refs.objects.get_descendants_count(id=item_id)
+        settings.LOCKS.acquire('ItemManager.delete_recursive')
+        item_lotsman_document_cnt = 0
+        cnt = Item_refs.objects.get_descendants_count(id=item_id)
 
-            with managed_progress(
-                    id=f'delete_recursive_{item_id}',
-                    qty=cnt,
-                    user=user,
-                    message='Удаление товарной(ых) позиций',
-                    title='Выполнено'
-            ) as progress:
+        with managed_progress(
+                id=f'delete_recursive_{item_id}',
+                qty=cnt,
+                user=user,
+                message='Удаление товарной(ых) позиций',
+                title='Выполнено',
+                props=props
+        ) as progress:
 
-                try:
-                    item = Item.objects.get(id=item_id)
-                    if cnt == 0:
-                        item.delete()
-                        return 1
+            try:
+                with transaction.atomic():
+                    try:
+                        item = Item.objects.get(id=item_id)
+                        if cnt == 0:
+                            Item_refs.objects1.filter(parent=item).delete_with_checked()
+                            item.delete()
+                            settings.LOCKS.release('ItemManager.delete_recursive')
+                            return 1
 
-                    _item = None
-                    progress.setContentsLabel(f'''<h3>Удаление товарной позиции ({item.STMP_1.value_str if item.STMP_1 else ''} : {item.STMP_2.value_str if item.STMP_2 else ''}).</h3>''')
-                    for item in Item_refs.objects.get_descendants(id=item_id, order_by_clause='order by level desc'):
-                        _item = item
-                        if not soft_delete:
-                            Item_image_refs.objects.filter(item_id=item.child_id).delete()
+                        _item = None
+                        progress.setContentsLabel(f'''<h3>Удаление товарной позиции ({item.STMP_1.value_str if item.STMP_1 else ''} : {item.STMP_2.value_str if item.STMP_2 else ''}).</h3>''')
+                        for item in Item_refs.objects.get_descendants(id=item_id, order_by_clause='order by level desc'):
+                            _item = item
+                            if not soft_delete:
+                                Item_image_refs.objects.filter(item_id=item.child_id).delete()
 
-                            if delete_lines:
-                                Item_line.objects.filter(parent_id=item.child_id).delete()
-                                Item_line.objects.filter(child_id=item.child_id).delete()
+                                if delete_lines:
+                                    Item_line.objects.filter(parent_id=item.child_id).delete()
+                                    Item_line.objects.filter(child_id=item.child_id).delete()
 
-                            Item_refs.objects.filter(parent_id=item.child_id).delete()
-                            Item_refs.objects.filter(child_id=item.child_id).delete()
+                                Item_refs.objects1.filter(parent_id=item.child_id).delete_with_checked()
+                                Item_refs.objects1.filter(child_id=item.child_id).delete_with_checked()
 
-                            Operations_item.objects.filter(item=item.child_id).delete()
-                            # Ready_2_launch_detail.objects.filter(item=item.child_id).delete()
+                                Operations_item.objects.filter(item=item.child_id).delete()
+                                # Ready_2_launch_detail.objects.filter(item=item.child_id).delete()
 
-                            Item.objects.filter(id=item.child_id).delete()
-                        else:
-                            Item_image_refs.objects.filter(item_id=item.child_id).soft_delete()
+                                Item.objects.filter(id=item.child_id).delete()
+                            else:
+                                Item_image_refs.objects.filter(item_id=item.child_id).soft_delete()
 
-                            if delete_lines:
-                                Item_line.objects.filter(parent_id=item.child_id).soft_delete()
-                                Item_line.objects.filter(child_id=item.child_id).soft_delete()
+                                if delete_lines:
+                                    Item_line.objects.filter(parent_id=item.child_id).soft_delete()
+                                    Item_line.objects.filter(child_id=item.child_id).soft_delete()
 
-                            Item_refs.objects.filter(parent_id=item.child_id).soft_delete()
-                            Item_refs.objects.filter(child_id=item.child_id).soft_delete()
-                            Item.objects.filter(id=item.child_id).soft_delete()
+                                Item_refs.objects.filter(parent_id=item.child_id).soft_delete()
+                                Item_refs.objects.filter(child_id=item.child_id).soft_delete()
+                                Item.objects.filter(id=item.child_id).soft_delete()
 
-                        if progress:
-                            progress.step()
-                except Item.DoesNotExist:
-                    pass
-                except db.models.deletion.ProtectedError as ex:
-                    logger.error(f'Item_refs: {item}')
-                    raise ex
+                            step_res = progress.step()
+                            if step_res != 0:
+                                raise ProgressDroped(progress_deleted)
+                    except Item.DoesNotExist:
+                        settings.LOCKS.release('ItemManager.delete_recursive')
 
-                if item_lotsman_document_cnt > 0:
-                    refresh_mat_view('kd_lotsman_documents_hierarcy_mview')
+            except db.models.deletion.ProtectedError as ex:
+                logger.error(f'Item_refs: {item}')
+                settings.LOCKS.release('ItemManager.delete_recursive')
+                raise ex
+
+            except ProgressDroped as ex:
+                Deleted_progresses.objects.filter(id_progress=progress.id, user=progress.user).delete()
+                settings.LOCKS.release('ItemManager.delete_recursive')
+                raise ex
+
+        settings.LOCKS.release('ItemManager.delete_recursive')
+
+        if item_lotsman_document_cnt > 0:
+            refresh_mat_view('kd_lotsman_documents_hierarcy_mview')
 
     def updateFromRequest(self, request, removed=None, function=None):
         from kaf_pas.ckk.models.item_refs import Item_refs
@@ -214,7 +229,9 @@ class ItemManager(CommonManagetWithLookUpFieldsManager):
 
             else:
                 def rec(data):
+                    _data = data.copy()
                     relevant = data.get('relevant')
+                    confirmed = data.get('confirmed')
                     props = data.get('props')
 
                     if relevant == 'Актуален':
@@ -222,6 +239,11 @@ class ItemManager(CommonManagetWithLookUpFieldsManager):
                     else:
                         props &= ~Item.props.relevant
 
+                    if confirmed == 'Подтвержден':
+                        props |= Item.props.confirmed
+                    else:
+                        props &= ~Item.props.confirmed
+
                     where_from = data.get('where_from')
                     if where_from == 'Получено из чертежа':
                         props |= Item.props.from_cdw
@@ -233,13 +255,25 @@ class ItemManager(CommonManagetWithLookUpFieldsManager):
                         props |= Item.props.man_input
 
                     delAttr(data, 'relevant')
+                    delAttr(data, 'confirmed')
                     delAttr(data, 'where_from')
                     delAttr(data, 'qty_operations')
                     delAttr(data, 'refs_props')
                     delAttr(data, 'icon')
+                    delAttr(data, 'id')
+                    delAttr(data, 'STMP_1__value_str')
+                    delAttr(data, 'STMP_2__value_str')
+                    delAttr(data, 'document__file_document')
+                    delAttr(data, 'isFolder')
+                    delAttr(data, 'isLotsman')
+                    delAttr(data, 'parent_id')
+                    delAttr(data, 'section')
+
                     setAttr(data, 'props', props)
 
-                    return self.filter(id=data.get('id')).update(**data)
+                    res = Item.objects.filter(id=_data.get('id')).update(**data)
+
+                    return _data
 
                 if isinstance(data, dict):
                     for k, _data in data.items():
@@ -247,6 +281,7 @@ class ItemManager(CommonManagetWithLookUpFieldsManager):
                             res = rec(_data)
                         else:
                             res = rec(data.copy())
+                            break
                 else:
                     res = 0
 
@@ -265,6 +300,7 @@ class ItemManager(CommonManagetWithLookUpFieldsManager):
             operations = get_operations_from_trunsaction(data)
             if isinstance(operations, list):
                 for operation in get_operations_from_trunsaction(data):
+                    settings.LOCKS.acquire('ItemManager.deleteFromRequest')
                     _data = operation.get('data')
                     if _data.get('mode') == 'deleteInner':
                         mode = 'deleteInner'
@@ -272,22 +308,27 @@ class ItemManager(CommonManagetWithLookUpFieldsManager):
 
                         if isinstance(records, list):
                             for record in records:
-                                res, _ = Item_refs.objects.filter(child_id=record.get('id'), parent_id=record.get('parent_id')).delete()
+                                res, _ = Item_refs.objects1.filter(child_id=record.get('id'), parent_id=record.get('parent_id')).delete_with_checked()
+                    settings.LOCKS.release('ItemManager.deleteFromRequest')
 
                 if mode == 'deleteInner':
                     return res
             elif isinstance(data, dict):
                 if data.get('mode') == 'deleteInner':
+                    settings.LOCKS.acquire('ItemManager.deleteFromRequest')
                     records = data.get('records')
 
                     if isinstance(records, list):
                         for record in records:
-                            res, _ = Item_refs.objects.filter(child_id=record.get('id'), parent_id=record.get('parent_id')).delete()
+                            res, _ = Item_refs.objects1.filter(child_id=record.get('id'), parent_id=record.get('parent_id')).delete_with_checked()
+                        settings.LOCKS.release('ItemManager.deleteFromRequest')
                         return res
+                    settings.LOCKS.release('ItemManager.deleteFromRequest')
                 elif data.get('mode') == 'reloadRefs':
                     records = data.get('records')
                     res = 0
                     if isinstance(records, list):
+                        settings.LOCKS.acquire('ItemManager.deleteFromRequest')
                         for record in records:
                             id = record.get('id')
                             parent_id = record.get('parent_id')
@@ -295,15 +336,52 @@ class ItemManager(CommonManagetWithLookUpFieldsManager):
                             for item_line in Item_line.objects.filter(parent_id=id):
                                 item_refs, _ = Item_refs.objects.get_or_create(parent_id=id, child=item_line.child)
                                 res += 1
+                        settings.LOCKS.release('ItemManager.deleteFromRequest')
 
                     return res
 
-        with transaction.atomic():
-            for id, mode in request.get_tuple_ids():
-                if mode == 'hide':
-                    res = self.delete_recursive(item_id=id, soft_delete=True, delete_lines=True, user=request.user_id)
-                else:
-                    res = self.delete_recursive(item_id=id, soft_delete=False, delete_lines=True, user=request.user_id)
+        for id, mode in request.get_tuple_ids():
+            if mode == 'hide':
+                res = self.delete_recursive(item_id=id, soft_delete=True, delete_lines=True, user=request.user_id)
+            else:
+                res = self.delete_recursive(item_id=id, soft_delete=False, delete_lines=True, user=request.user_id)
+        return res
+
+    def lookupFromRequest(self, request):
+        request = DSRequest(request=request)
+        data = request.get_data()
+
+        attr_type__code = data.get('attr_type__code')
+        value_str = data.get(f'{attr_type__code}__value_str')
+        value_int = data.get(f'{attr_type__code}__value_int')
+
+        if value_str == None and value_int != None:
+            value_str = str(value_int)
+
+        if value_str == None:
+            return None
+
+        res, created = Document_attributesManager.get_or_create_attribute(attr_codes=attr_type__code, value_str=value_str)
+        res = model_to_dict(res)
+        if res.get('value_str') == 'null':
+            setAttr(res, 'value_str', None)
+        return res
+
+    @staticmethod
+    def get_verstion(STMP_1, STMP_2, props, inc=1):
+        settings.LOCKS.acquire('ItemManager.get_verstion')
+        try:
+            res = Item.objects.filter(
+                STMP_1_id=STMP_1.id if STMP_1 else None,
+                STMP_2_id=STMP_2.id if STMP_2 else None,
+                props=props,
+                version__isnull=False
+            ).order_by('-version')[0].version + inc
+            if res == None:
+                res = 1
+        except IndexError:
+            res = 1
+        settings.LOCKS.release('ItemManager.get_verstion')
         return res
 
     @staticmethod
@@ -328,6 +406,8 @@ class ItemManager(CommonManagetWithLookUpFieldsManager):
 
     def checkRecursives(self, request, ):
         from kaf_pas.ckk.management.commands.check_item_cicular_refs import check_level
+        from isc_common.models.deleted_progresses import Deleted_progresses
+
         request = DSRequest(request=request)
         data = request.get_data()
         if isinstance(data, dict) and isinstance(data.get('records'), list):
@@ -340,13 +420,17 @@ class ItemManager(CommonManagetWithLookUpFieldsManager):
                     title='Выполнено',
                     props=TurnBitOn(0, 0)
             ) as progress:
-                for record in data.get('records'):
-                    cycle_ref.extend(check_level(parent_array=list(), item=Item.objects.get(id=record.get('id')), level=1, table='Item_refs', progress=progress))
-                if len(cycle_ref) == 0:
-                    progress.sendInfo('Циклических зависемостей не найдено.')
-                else:
-                    progress.sendInfo('\n'.join(cycle_ref))
-                return dict(messages=cycle_ref)
+                try:
+                    for record in data.get('records'):
+                        cycle_ref.extend(check_level(parent_array=list(), item=Item.objects.get(id=record.get('id')), level=1, table='Item_refs', progress=progress))
+                    if len(cycle_ref) == 0:
+                        progress.sendInfo('Циклических зависемостей не найдено.')
+                    else:
+                        progress.sendInfo('\n'.join(cycle_ref))
+                    return dict(messages=cycle_ref)
+                except ProgressDroped as ex:
+                    Deleted_progresses.objects.filter(id_progress=progress.id, user=progress.user).delete()
+                    raise ex
         raise Exception(f'Not data.')
 
 
@@ -362,6 +446,7 @@ class Item_add:
             ('man_input', 'Занесено вручную'),  # 32
             ('copied', 'Скопировано'),  # 64
             ('from_lotsman', 'Получено из Лоцмана'),  # 128
+            ('confirmed', 'Подтвержден'),  # 256
         ), default=1, db_index=True)
 
 
@@ -395,15 +480,6 @@ class Item(AuditModel):
             return None
         return doc_attr.value_str.strip() if doc_attr else None
 
-    def save(self, *args, **kwargs):
-        if ~self.props.relevant:
-            item = Item.objects.filter()
-            from django.db.models import Max
-
-            res = item.aggregate(Max('version'))
-            self.version = 1 if not res.get('version__max') else res.get('version__max') + 1
-        super().save(*args, **kwargs)
-
     def __str__(self):
         return f'ID={self.id} STMP_1=[{self.STMP_1}], STMP_2=[{self.STMP_2}], props={self.props}, version={self.version}'
 
