@@ -3,8 +3,10 @@ import uuid
 
 from django.db import transaction
 from django.db.models import PositiveIntegerField, CheckConstraint, Q, F
+from django.forms import model_to_dict
 
 from isc_common import setAttr, delAttr
+from isc_common.bit import TurnBitOn
 from isc_common.fields.code_field import CodeField
 from isc_common.fields.name_field import NameField
 from isc_common.fields.related import ForeignKeyProtect
@@ -12,10 +14,10 @@ from isc_common.http.DSRequest import DSRequest
 from isc_common.models.audit import AuditModel, AuditQuerySet
 from isc_common.models.tree_audit import TreeAuditModelManager
 from isc_common.number import StrToNumber
-from isc_common.progress import managed_progress
+from isc_common.progress import managed_progress, ProgressDroped
 from kaf_pas.ckk.models.item import Item
 from kaf_pas.ckk.models.item_refs import Item_refs
-from kaf_pas.kd.models.document_attributes import Document_attributes
+from kaf_pas.kd.models.document_attributes import Document_attributes, Document_attributesManager
 
 logger = logging.getLogger(__name__)
 
@@ -211,6 +213,7 @@ class Item_lineManager(TreeAuditModelManager):
         request = DSRequest(request=request)
         id = request.get_id()
         data = request.get_data()
+        data_res = request.get_data()
         mode = data.get('mode')
 
         if mode == 'item_spw_replace':
@@ -230,7 +233,11 @@ class Item_lineManager(TreeAuditModelManager):
                 targetLine = Item_line.objects.get(child_id=targetRecord.get('child_id'), parent_id=targetRecord.get('parent_id'))
                 dropItem = Item.objects.get(id=dropRecord.get('id'))
 
-                Item_refs.objects.get(child=targetLine.child, parent_id=targetLine.parent).delete()
+                try:
+                    Item_refs.objects.get(child=targetLine.child, parent_id=targetLine.parent).delete()
+                except Item_refs.DoesNotExist:
+                    pass
+
                 targetLine.child_id = dropItem.id
                 targetLine.SPC_CLM_NAME = dropItem.STMP_1
                 setAttr(targetRecord, 'child_id', dropItem.id)
@@ -245,6 +252,15 @@ class Item_lineManager(TreeAuditModelManager):
 
                 return targetRecord
 
+        relevant = data.get('relevant')
+        item_props = data.get('item_props')
+        child_id = data.get('child_id')
+
+        if relevant == 'Актуален':
+            item_props |= Item.props.relevant
+        elif relevant == 'Не актуален':
+            item_props &= ~Item.props.relevant
+
         delAttr(data, 'document_id')
         delAttr(data, 'isFolder')
         delAttr(data, 'icon')
@@ -257,53 +273,80 @@ class Item_lineManager(TreeAuditModelManager):
         _data = dict()
 
         for key, val in data.items():
-            if str(key).find('_value_str') == -1 and str(key).find('_value_int') == -1:
+            if str(key).find('__value_str') == -1 and str(key).find('__value_int') == -1:
                 setAttr(_data, key, val)
 
-        res = super().filter(id=id).update(**_data)
-        return data
+        with transaction.atomic():
+            res = super().filter(id=id).update(**_data)
+            Item.objects.update_or_create(id=child_id, defaults=dict(props=item_props))
+        return data_res
 
     def copyFromRequest(self, request, removed=None, ):
         from kaf_pas.ckk.models.item_line_view import Item_line_view
+        from isc_common.models.deleted_progresses import Deleted_progresses
 
         request = DSRequest(request=request)
-        res = 0
 
-        with transaction.atomic():
-            data = request.json.get('data')
-            with managed_progress(
-                    qty=len(data.get('records')),
-                    id=str(uuid.uuid4()),
-                    user=request.user,
-                    title="Выполнено"
-            ) as progress:
-                progress.setContentsLabel(f'<b>Создание строк детализации</b>')
-                for record in data.get('records'):
-                    for item_line in Item_line_view.objects.filter(
-                            SPC_CLM_NAME__value_str=record.get('STMP_1__value_str'),
-                            SPC_CLM_MARK__value_str=record.get('STMP_2__value_str')
-                    ):
-                        res, _ = Item_line.objects.get_or_create(
-                            parent_id=record.get("parent_id"),
-                            child_id=record.get("id"),
-                            SPC_CLM_FORMAT=item_line.SPC_CLM_FORMAT,
-                            SPC_CLM_ZONE=item_line.SPC_CLM_ZONE,
-                            SPC_CLM_POS=item_line.SPC_CLM_POS,
-                            SPC_CLM_MARK=item_line.SPC_CLM_MARK,
-                            SPC_CLM_NAME=item_line.SPC_CLM_NAME,
-                            SPC_CLM_COUNT=item_line.SPC_CLM_COUNT,
-                            SPC_CLM_NOTE=item_line.SPC_CLM_NOTE,
-                            SPC_CLM_MASSA=item_line.SPC_CLM_MASSA,
-                            SPC_CLM_MATERIAL=item_line.SPC_CLM_MATERIAL,
-                            SPC_CLM_USER=item_line.SPC_CLM_USER,
-                            SPC_CLM_KOD=item_line.SPC_CLM_USER,
-                            SPC_CLM_FACTORY=item_line.SPC_CLM_FACTORY,
-                            section=item_line.section,
-                            section_num=item_line.section_num,
-                            subsection=item_line.subsection,
-                        )
-                        break
-                    progress.step()
+        data = request.json.get('data')
+        with managed_progress(
+                qty=len(data.get('records')),
+                id=str(uuid.uuid4()),
+                user=request.user,
+                title="Выполнено",
+                props=TurnBitOn(0, 0)
+        ) as progress:
+            try:
+                with transaction.atomic():
+                    progress.setContentsLabel(f'<b>Создание строк детализации</b>')
+                    for record in data.get('records'):
+                        for item_line in Item_line_view.objects.filter(
+                                SPC_CLM_NAME__value_str=record.get('STMP_1__value_str'),
+                                SPC_CLM_MARK__value_str=record.get('STMP_2__value_str')
+                        ):
+                            res, _ = Item_line.objects.get_or_create(
+                                parent_id=record.get("parent_id"),
+                                child_id=record.get("id"),
+                                SPC_CLM_FORMAT=item_line.SPC_CLM_FORMAT,
+                                SPC_CLM_ZONE=item_line.SPC_CLM_ZONE,
+                                SPC_CLM_POS=item_line.SPC_CLM_POS,
+                                SPC_CLM_MARK=item_line.SPC_CLM_MARK,
+                                SPC_CLM_NAME=item_line.SPC_CLM_NAME,
+                                SPC_CLM_COUNT=item_line.SPC_CLM_COUNT,
+                                SPC_CLM_NOTE=item_line.SPC_CLM_NOTE,
+                                SPC_CLM_MASSA=item_line.SPC_CLM_MASSA,
+                                SPC_CLM_MATERIAL=item_line.SPC_CLM_MATERIAL,
+                                SPC_CLM_USER=item_line.SPC_CLM_USER,
+                                SPC_CLM_KOD=item_line.SPC_CLM_USER,
+                                SPC_CLM_FACTORY=item_line.SPC_CLM_FACTORY,
+                                section=item_line.section,
+                                section_num=item_line.section_num,
+                                subsection=item_line.subsection,
+                            )
+                            break
+                        progress.step()
+            except ProgressDroped as ex:
+                Deleted_progresses.objects.filter(id_progress=progress.id, user=progress.user).delete()
+                raise ex
+
+    def lookupFromRequest(self, request):
+        request = DSRequest(request=request)
+        data = request.get_data()
+
+        attr_type__code = data.get('attr_type__code')
+        value_str = data.get(f'{attr_type__code}__value_str')
+        value_int = data.get(f'{attr_type__code}__value_int')
+
+        if value_str == None and value_int != None:
+            value_str = str(value_int)
+
+        if value_str == None:
+            return None
+
+        res, created = Document_attributesManager.get_or_create_attribute(attr_codes=attr_type__code, value_str=value_str)
+        res = model_to_dict(res)
+        if res.get('value_str') == 'null':
+            setAttr(res, 'value_str', None)
+        return res
 
     def deleteFromRequest(self, request, removed=None, ):
         from kaf_pas.ckk.models.item_refs import Item_refs
