@@ -1,5 +1,6 @@
 import logging
 
+from django.conf import settings
 from django.db import transaction
 from django.db.models import BigIntegerField, BooleanField, PositiveIntegerField
 from django.forms import model_to_dict
@@ -85,6 +86,8 @@ class Item_viewManager(TreeAuditModelManager):
         with transaction.atomic():
 
             relevant = data.get('relevant')
+            confirmed = data.get('confirmed')
+
             props = data.get('props', 0)
             refs_props = data.get('refs_props', 0)
             refs_id = data.get('refs_id')
@@ -94,6 +97,11 @@ class Item_viewManager(TreeAuditModelManager):
             else:
                 props &= ~Item.props.relevant
 
+            if confirmed == 'Подтвержден':
+                props |= Item.props.confirmed
+            else:
+                props &= ~Item.props.confirmed
+
             where_from = data.get('where_from')
             if where_from == 'Получено из чертежа':
                 props |= Item.props.from_cdw
@@ -105,6 +113,7 @@ class Item_viewManager(TreeAuditModelManager):
                 props |= Item.props.man_input
 
             delAttr(data, 'relevant')
+            delAttr(data, 'confirmed')
             delAttr(data, 'where_from')
             delAttr(data, 'qty_operations')
             delAttr(data, 'refs_props')
@@ -133,6 +142,7 @@ class Item_viewManager(TreeAuditModelManager):
             "deliting": record.deliting,
             "isFolder": record.isFolder,
             "relevant": record.relevant,
+            "confirmed": record.confirmed,
             "section": record.section,
             "version": record.version,
             "where_from": record.where_from,
@@ -156,6 +166,7 @@ class Item_viewManager(TreeAuditModelManager):
         if not isinstance(item, Item_view) and not isinstance(item, Item):
             raise Exception(f'item must be a Item type')
 
+        settings.LOCKS.acquire('Item_viewManager.copy_item')
         dict_item = model_to_dict(item)
         delAttr(dict_item, 'id')
         delAttr(dict_item, 'relevant')
@@ -216,6 +227,7 @@ class Item_viewManager(TreeAuditModelManager):
             setAttr(dict_item_image_refs, 'thumb10_id', dict_item_image_refs.get('thumb10'))
             delAttr(dict_item_image_refs, 'thumb10')
             Item_image_refs.objects.get_or_create(**dict_item_image_refs)
+        settings.LOCKS.release('Item_viewManager.copy_item')
 
         return new_item
 
@@ -242,6 +254,7 @@ class Item_view(Hierarcy):
     refs_id = BigIntegerField()
     isLotsman = BooleanField()
     relevant = NameField()
+    confirmed = NameField()
     where_from = NameField()
     props = Item_add.get_prop_field()
     refs_props = Item_refsManager.props()
